@@ -36,7 +36,7 @@ hexdump(covio_t *io, off_t lastoff)
     io->seek(lastoff);
     for ( ; lastoff < here ; lastoff += 4)
     {
-    	io->read_u32(&d);
+    	io->read_u32(d);
 	printf(GNB_U32_XFMT" ", d);
     }
     assert(here == lastoff);
@@ -49,29 +49,27 @@ hexdump(covio_t *io, off_t lastoff)
 #define BB_ENDOFLIST	0x00000000
 
 static void
-do_tags(covio_old_t *io)
+do_tags(covio_t *io)
 {
     gnb_u32_t tag;
     off_t lastoff = 0;
-    char *s;
+    estring s;
     
-    while (io->read_u32(&tag))
+    while (io->read_u32(tag))
     {
     	printf("%08lx: ", lastoff);
     	switch (tag)
 	{
 	case BB_FILENAME:
-	    s = io->read_bbstring(tag);
+	    io->read_bbstring(s, tag);
 	    hexdump(io, lastoff);
-	    printf("file \"%s\"\n", s);
-	    g_free(s);
+	    printf("file \"%s\"\n", s.data());
 	    break;
 	
 	case BB_FUNCTION:
-	    s = io->read_bbstring(tag);
+	    io->read_bbstring(s, tag);
 	    hexdump(io, lastoff);
-	    printf("func \"%s\"\n", s);
-	    g_free(s);
+	    printf("func \"%s\"\n", s.data());
 	    break;
 	
 	case BB_ENDOFLIST:
@@ -91,13 +89,14 @@ do_tags(covio_old_t *io)
 static void
 do_file(const char *filename)
 {
-    covio_old_t io(filename);
+    covio_t io(filename);
     
     if (!io.open_read())
     {
     	perror(filename);
 	return;
     }
+    io.set_format(covio_t::FORMAT_OLD);
     
     do_tags(&io);
 }
