@@ -25,7 +25,7 @@
 #include <dirent.h>
 #include <fcntl.h>
 
-CVSID("$Id: filename.c,v 1.9 2005/05/18 13:05:13 gnb Exp $");
+CVSID("$Id: filename.c,v 1.10 2006/01/29 23:39:01 gnb Exp $");
 
 #ifndef __set_errno
 #define __set_errno(v)	 errno = (v)
@@ -147,6 +147,15 @@ file_open_mode(const char *filename, const char *rw, mode_t mode)
  * Return a new string which is a normalised absolute filename.
  */
 
+static gboolean
+is_path_tail(const char *path, const char *file)
+{
+    if (strlen(file) > strlen(path))
+    	return FALSE;
+    const char *tail = path + strlen(path) - strlen(file);
+    return (!strcmp(tail, file) && (tail == path || tail[-1] == '/'));
+}
+
 char *
 file_make_absolute_to(const char *filename, const char *absfile)
 {
@@ -160,6 +169,8 @@ file_make_absolute_to(const char *filename, const char *absfile)
     }
     else if (absfile != 0)
     {
+    	if (is_path_tail(absfile, filename))
+	    return g_strdup(absfile);
     	const char *t = strrchr(absfile, '/');
 	assert(t != 0);
     	abs.append_chars(absfile, t-absfile);
