@@ -30,16 +30,26 @@
 #include "fakepopt.h"
 #include <db.h>
 
-CVSID("$Id: ggcov-webdb.c,v 1.5 2005/06/13 07:29:11 gnb Exp $");
+CVSID("$Id: ggcov-webdb.c,v 1.10 2006/01/29 22:52:48 gnb Exp $");
+
+#define V(major,minor,patch)    ((major)*10000+(minor)*1000+(patch))
+#define DB_VERSION_CODE V(DB_VERSION_MAJOR,DB_VERSION_MINOR,DB_VERSION_PATCH)
+
+/*
+ * The transaction argument to DB->open was added in 4.1.25.
+ * As ggcov doesn't use transactions this define allows us
+ * to compile with older DB versions (some distros still
+ * ship db3.3.x as the default DB version).
+ */
+#if DB_VERSION_CODE < V(4,1,25)
+#define OPEN_TXN
+#else
+#define OPEN_TXN    (DB_TXN *)0,
+#endif
 
 char *argv0;
-GList *files;	    /* incoming specification from commandline */
+static GList *files;	    /* incoming specification from commandline */
 
-static int recursive = FALSE;	/* needs to be int (not gboolean) for popt */
-static char *suppressed_ifdefs = 0;
-static char *object_dir = 0;
-static int solve_fuzzy_flag = FALSE;
-static const char *debug_str = 0;
 static char *dump_mode = NULL;
 static char *output_tarball = "ggcov.webdb.tgz";
 
@@ -130,83 +140,6 @@ systemf(const char *fmt, ...)
     return system(cmd.data());
 }
 
-/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
-/* 
- * TODO: This function should be common with the identical code in ggcov.c
- */
-
-static void
-read_gcov_files(void)
-{
-    GList *iter;
-    
-    cov_init();
-
-    cov_function_t::set_solve_fuzzy_flag(solve_fuzzy_flag);
-
-    if (suppressed_ifdefs != 0)
-    {
-    	tok_t tok(/*force copy*/(const char *)suppressed_ifdefs, ", \t");
-	const char *v;
-	
-	while ((v = tok.next()) != 0)
-    	    cov_suppress_ifdef(v);
-    }
-
-    cov_pre_read();
-    
-    if (object_dir != 0)
-    	cov_add_search_directory(object_dir);
-
-    if (files == 0)
-    {
-    	if (!cov_read_directory(".", recursive))
-	    exit(1);
-    }
-    else
-    {
-	for (iter = files ; iter != 0 ; iter = iter->next)
-	{
-	    const char *filename = (const char *)iter->data;
-	    
-	    if (file_is_directory(filename) == 0)
-	    	cov_add_search_directory(filename);
-    	}
-
-	for (iter = files ; iter != 0 ; iter = iter->next)
-	{
-	    const char *filename = (const char *)iter->data;
-	    
-	    if (file_is_directory(filename) == 0)
-	    {
-	    	if (!cov_read_directory(filename, recursive))
-		    exit(1);
-	    }
-	    else if (file_is_regular(filename) == 0)
-	    {
-	    	if (cov_is_source_filename(filename))
-		{
-		    if (!cov_read_source_file(filename))
-			exit(1);
-		}
-		else
-		{
-		    if (!cov_read_object_file(filename))
-			exit(1);
-		}
-	    }
-	    else
-	    {
-	    	fprintf(stderr, "%s: don't know how to handle this filename\n",
-		    	filename);
-		exit(1);
-	    }
-	}
-    }
-    
-    cov_post_read();
-}
-
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
 static void
@@ -974,34 +907,6 @@ create_source_symlinks(const char *tempdir)
 static poptContext popt_context;
 static struct poptOption popt_options[] =
 {
-    {
-    	"recursive",	    	    	    	/* longname */
-	'r',  	    	    	    	    	/* shortname */
-	POPT_ARG_NONE,  	    	    	/* argInfo */
-	&recursive,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"recursively scan directories for source", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    {
-    	"suppress-ifdef",	    	    	/* longname */
-	'X',  	    	    	    	    	/* shortname */
-	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&suppressed_ifdefs,     	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"suppress source which is conditional on the "
-	"given cpp define/s (comma-separated)", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    {
-    	"object-directory",    	    	    	/* longname */
-	'o',  	    	    	    	    	/* shortname */
-	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&object_dir,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"directory in which to find .o,.bb,.bbg,.da files", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
     {
     	"output-file",    	    	    	/* longname */
 	'f',  	    	    	    	    	/* shortname */
@@ -1020,26 +925,9 @@ static struct poptOption popt_options[] =
 	"dump the entire database",		/* descrip */
 	0	    	    	    	    	/* argDescrip */
     },
-    {
-    	"solve-fuzzy",    	    	    	/* longname */
-	'F',  	    	    	    	    	/* shortname */
-	POPT_ARG_NONE,  	    	    	/* argInfo */
-	&solve_fuzzy_flag,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"whether to be tolerant of inconsistent arc counts", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    {
-    	"debug",	    	    	    	/* longname */
-	'D',  	    	    	    	    	/* shortname */
-	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&debug_str,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"enable tggcov debugging features",  	/* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
+    COV_POPT_OPTIONS
     POPT_AUTOHELP
-    { 0, 0, 0, 0, 0, 0, 0 }
+    POPT_TABLEEND
 };
 
 static void
@@ -1070,25 +958,8 @@ parse_args(int argc, char **argv)
 	files = g_list_append(files, (gpointer)file);
 	
     poptFreeContext(popt_context);
-    
-    if (debug_str != 0)
-    	debug_set(debug_str);
 
-    if (debug_enabled(D_DUMP|D_VERBOSE))
-    {
-    	GList *iter;
-	string_var token_str = debug_enabled_tokens();
-
-	duprintf1("parse_args: recursive=%d\n", recursive);
-	duprintf1("parse_args: suppressed_ifdefs=%s\n", suppressed_ifdefs);
-	duprintf1("parse_args: solve_fuzzy_flag=%d\n", solve_fuzzy_flag);
-	duprintf2("parse_args: debug = 0x%lx (%s)\n", debug, token_str.data());
-
-	duprintf0("parse_args: files = ");
-	for (iter = files ; iter != 0 ; iter = iter->next)
-	    duprintf1(" \"%s\"", (char *)iter->data);
-	duprintf0(" }\n");
-    }
+    cov_post_args();
 }
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
@@ -1142,7 +1013,7 @@ create_database(void)
     DB *db;
     int ret;
 
-    read_gcov_files();
+    cov_read_files(files);
 
     tempdir = g_strdup_printf("%s/ggcov-web-%d.d", get_tmpdir(), (int)getpid());
     webdb_file = g_strconcat(tempdir, "/", "ggcov.webdb", (char*)0);
@@ -1152,13 +1023,13 @@ create_database(void)
     systemf("/bin/rm -rf \"%s\"", tempdir);
     file_build_tree(tempdir, 0755);
 
-    if (ret = db_create(&db, 0, 0))
+    if ((ret = db_create(&db, 0, 0)))
     {
 	fprintf(stderr, "%s: db_create(): %s\n", argv0, db_strerror(ret));
 	exit(1);
     }
 
-    ret = db->open(db, 0, webdb_file, 0, DB_HASH, DB_CREATE, 0644);
+    ret = db->open(db, OPEN_TXN webdb_file, 0, DB_HASH, DB_CREATE, 0644);
     if (ret)
     {
 	db->err(db, ret, "%s", webdb_file);
@@ -1232,7 +1103,7 @@ dump_database(const char *mode)
 	exit(1);
     }
 
-    if ((ret = db->open(db, 0, webdb_file, 0, DB_HASH, DB_CREATE, 0644)))
+    if ((ret = db->open(db, OPEN_TXN webdb_file, 0, DB_HASH, DB_CREATE, 0644)))
     {
 	db->err(db, ret, "%s", webdb_file);
 	exit(1);
