@@ -37,19 +37,15 @@
 #include <libgnomeui/libgnomeui.h>
 #include "fakepopt.h"
 
-CVSID("$Id: ggcov.c,v 1.47 2005/06/13 07:08:51 gnb Exp $");
+CVSID("$Id: ggcov.c,v 1.52 2006/06/26 03:00:39 gnb Exp $");
 
 #define DEBUG_GTK 1
 
 char *argv0;
-GList *files;	    /* incoming specification from commandline */
+static GList *files;	    /* incoming specification from commandline */
 
-static int recursive = FALSE;	/* needs to be int (not gboolean) for popt */
-static char *suppressed_ifdefs = 0;
-static char *object_dir = 0;
-static const char *debug_str = 0;
 static const char ** debug_argv;
-static int solve_fuzzy_flag = FALSE;
+static const char *initial_windows = "summary";
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 /*
@@ -85,77 +81,6 @@ stash_argv(int argc, char **argv)
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
-/*
- * Read files from the commandline.
- */
-static void
-read_gcov_files(void)
-{
-    GList *iter;
-    
-    cov_init();
-    if (files == 0)
-    	return;
-
-    cov_function_t::set_solve_fuzzy_flag(solve_fuzzy_flag);
-
-    if (suppressed_ifdefs != 0)
-    {
-    	tok_t tok(/*force copy*/(const char *)suppressed_ifdefs, ", \t");
-	const char *v;
-	
-	while ((v = tok.next()) != 0)
-    	    cov_suppress_ifdef(v);
-    }
-
-    cov_pre_read();
-    
-    if (object_dir != 0)
-    	cov_add_search_directory(object_dir);
-
-    for (iter = files ; iter != 0 ; iter = iter->next)
-    {
-	const char *filename = (const char *)iter->data;
-
-	if (file_is_directory(filename) == 0)
-	    cov_add_search_directory(filename);
-    }
-
-    for (iter = files ; iter != 0 ; iter = iter->next)
-    {
-	const char *filename = (const char *)iter->data;
-
-	if (file_is_directory(filename) == 0)
-	{
-	    if (!cov_read_directory(filename, recursive))
-		exit(1);
-	}
-	else if (file_is_regular(filename) == 0)
-	{
-	    if (cov_is_source_filename(filename))
-	    {
-		if (!cov_read_source_file(filename))
-		    exit(1);
-	    }
-	    else
-	    {
-		if (!cov_read_object_file(filename))
-		    exit(1);
-	    }
-	}
-	else
-	{
-	    fprintf(stderr, "%s: don't know how to handle this filename\n",
-		    filename);
-	    exit(1);
-	}
-    }
-    
-    cov_post_read();
-}
-
-/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
-
 /*
  * Read a file from the File->Open dialog.
  */
@@ -164,7 +89,7 @@ ggcov_read_file(const char *filename)
 {
     if (file_is_directory(filename) == 0)
     {
-	if (!cov_read_directory(filename, recursive))
+	if (!cov_read_directory(filename, /*recursive*/FALSE))
 	    return FALSE;
     }
     else if (file_is_regular(filename) == 0)
@@ -340,8 +265,31 @@ on_windows_new_reportwin_activated(GtkWidget *w, gpointer userdata)
 #include "ui/ggcov32.xpm"
 
 static void
-ui_create(void)
+ui_create(const char *full_argv0)
 {
+    /*
+     * If we're being run from the source directory, fiddle the
+     * glade search path to point to ../ui first.  This means
+     * we can run ggcov before installation without having to
+     * compile with UI_DEBUG=1.
+     */
+    estring dir = file_make_absolute(full_argv0);
+    const char *p = strrchr(dir, '/');
+    if (p != 0 &&
+	(p -= 4) >= dir.data() &&
+	!strncmp(p, "/src/", 5))
+    {
+	dir.truncate_to(p - dir);
+	dir.append_string("/ui");
+	if (!file_is_directory(dir))
+	{
+	    fprintf(stderr, "ggcov: running from source directory, "
+			    "so prepending %s to glade search path\n",
+			    dir.data());
+	    ui_prepend_glade_path(dir);
+	}
+    }
+
     ui_register_windows_entry("New Summary...",
     			      on_windows_new_summarywin_activated, 0);
     ui_register_windows_entry("New File List...",
@@ -373,10 +321,40 @@ ui_create(void)
 	    gtk_main_iteration();
     }
 
-    /* Possibly have files from commandline or dialog...show the Summary window */
-    summarywin_t *sw = new summarywin_t();
-    sw->show();
-    prefs.post_load(sw->get_window());
+    /* Possibly have files from commandline or dialog...show initial windows */
+    tok_t tok(initial_windows, ", \n\r");
+    const char *name;
+    int nwindows = 0;
+    while ((name = tok.next()) != 0)
+    {
+	nwindows++;
+	if (!strcmp(name, "summary"))
+	    on_windows_new_summarywin_activated(0, 0);
+	else if (!strcmp(name, "files"))
+	    on_windows_new_fileswin_activated(0, 0);
+	else if (!strcmp(name, "functions"))
+	    on_windows_new_functionswin_activated(0, 0);
+	else if (!strcmp(name, "calls"))
+	    on_windows_new_callswin_activated(0, 0);
+	else if (!strcmp(name, "callbutterfly"))
+	    on_windows_new_callgraphwin_activated(0, 0);
+	else if (!strcmp(name, "callgraph"))
+	    on_windows_new_callgraph2win_activated(0, 0);
+	else if (!strcmp(name, "lego"))
+	    on_windows_new_legowin_activated(0, 0);
+	else if (!strcmp(name, "source"))
+	    on_windows_new_sourcewin_activated(0, 0);
+	else if (!strcmp(name, "reports"))
+	    on_windows_new_reportwin_activated(0, 0);
+	else
+	{
+	    fprintf(stderr, "%s: unknown window name \"%s\"\n", argv0, name);
+	    nwindows--;
+	}
+    }
+
+    if (!nwindows)
+	on_windows_new_summarywin_activated(0, 0);
 }
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
@@ -392,54 +370,20 @@ ui_create(void)
  * specification of options, i.e. we simulate popt in fakepopt.c!
  */
 static poptContext popt_context;
-static struct poptOption popt_options[] =
+static const struct poptOption popt_options[] =
 {
     {
-    	"recursive",	    	    	    	/* longname */
-	'r',  	    	    	    	    	/* shortname */
-	POPT_ARG_NONE,  	    	    	/* argInfo */
-	&recursive,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"recursively scan directories for source", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    {
-    	"suppress-ifdef",	    	    	/* longname */
-	'X',  	    	    	    	    	/* shortname */
+    	"initial-windows",    	    	    	/* longname */
+	'w',  	    	    	    	    	/* shortname */
 	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&suppressed_ifdefs,     	    	/* arg */
+	&initial_windows,     	    	    	/* arg */
 	0,  	    	    	    	    	/* val 0=don't return */
-	"suppress source which is conditional on this cpp define", /* descrip */
+	"list of windows to open initially",	/* descrip */
 	0	    	    	    	    	/* argDescrip */
     },
-    {
-    	"object-directory",    	    	    	/* longname */
-	'o',  	    	    	    	    	/* shortname */
-	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&object_dir,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"directory in which to find .o,.bb,.bbg,.da files", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    {
-    	"solve-fuzzy",    	    	    	/* longname */
-	'F',  	    	    	    	    	/* shortname */
-	POPT_ARG_NONE,  	    	    	/* argInfo */
-	&solve_fuzzy_flag,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"whether to be tolerant of inconsistent arc counts", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    {
-    	"debug",	    	    	    	/* longname */
-	'D',  	    	    	    	    	/* shortname */
-	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&debug_str,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"enable ggcov debugging features",  	/* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    { 0, 0, 0, 0, 0, 0, 0 }
+    COV_POPT_OPTIONS
+    POPT_AUTOHELP
+    POPT_TABLEEND
 };
 
 static void
@@ -449,7 +393,9 @@ parse_args(int argc, char **argv)
     
     argv0 = argv[0];
     
-#if GTK2
+#if HAVE_GNOME_PROGRAM_INIT
+    /* gnome_program_init() already has popt options */
+#elif GTK2
     popt_context = poptGetContext(PACKAGE, argc, (const char**)argv,
     	    	    	    	  popt_options, 0);
     int rc;
@@ -475,15 +421,11 @@ parse_args(int argc, char **argv)
 	
     poptFreeContext(popt_context);
     
-    if (debug_str != 0)
-    	debug_set(debug_str);
-	
-
+    cov_post_args();
+    
     if (debug_enabled(D_DUMP|D_VERBOSE))
     {
-    	GList *iter;
 	const char **p;
-	string_var token_str = debug_enabled_tokens();
 
 	duprintf0("parse_args: argv[] = {");
 	for (p = debug_argv ; *p ; p++)
@@ -494,15 +436,6 @@ parse_args(int argc, char **argv)
 	    	duprintf1(" \"%s\"", *p);
 	}
 	duprintf0(" }\n");
-
-	duprintf1("parse_args: recursive = %d\n", recursive);
-
-	duprintf2("parse_args: debug = 0x%lx (%s)\n", debug, token_str.data());
-
-	duprintf0("parse_args: files = {");
-	for (iter = files ; iter != 0 ; iter = iter->next)
-	    duprintf1(" \"%s\"", (char *)iter->data);
-	duprintf0(" }\n");
     }
 }
 
@@ -575,8 +508,10 @@ main(int argc, char **argv)
     
     prog = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE,
 			      argc, argv,
+			      GNOME_PARAM_POPT_TABLE, popt_options,
 			      GNOME_PROGRAM_STANDARD_PROPERTIES,
 			      GNOME_PARAM_NONE);
+    g_object_get(prog, GNOME_PARAM_POPT_CONTEXT, &popt_context, (char *)0);
 #elif GTK2
     gtk_init(&argc, &argv);
     /* As of 2.0 we don't need to explicitly initialise libGlade anymore */
@@ -588,10 +523,10 @@ main(int argc, char **argv)
 #endif
 
     parse_args(argc, argv);
-    read_gcov_files();
+    cov_read_files(files);
 
     cov_dump(stderr);
-    ui_create();
+    ui_create(argv[0]);
     gtk_main();
 
     return 0;
