@@ -23,7 +23,7 @@
 #include "string_var.H"
 #include "tok.H"
 
-CVSID("$Id: ui.c,v 1.29 2005/03/18 15:39:06 gnb Exp $");
+CVSID("$Id: ui.c,v 1.32 2006/06/27 13:31:13 gnb Exp $");
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
@@ -99,11 +99,7 @@ ui_combo_clear(GtkCombo *combo)
 #define UI_DEBUG 0
 #endif
 
-const char *ui_glade_path = 
-#if UI_DEBUG
-"ui:../../ui:"
-#endif
-PKGDATADIR;
+static string_var ui_glade_path = PKGDATADIR;
 
 static char *
 find_file(const char *path, const char *base)
@@ -124,6 +120,59 @@ find_file(const char *path, const char *base)
     return 0;
 }
 
+void
+ui_prepend_glade_path(const char *dir)
+{
+    ui_glade_path = g_strconcat(dir, ":", ui_glade_path.data(), 0);
+}
+
+/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
+
+extern struct { const char *name; GCallback func; } ui_callbacks[];   /* in glade_callbacks.c */
+
+/*
+ * Connect up named functions for signal handlers.
+ */
+static void
+ui_register_callbacks(GladeXML *xml)
+{
+    int i;
+
+    for (i = 0 ; ui_callbacks[i].name ; i++)
+	glade_xml_signal_connect(xml, ui_callbacks[i].name, ui_callbacks[i].func);
+}
+
+/*
+ * Unfortunately, we can't use glade_xml_signal_connect()
+ * to hook up our custom widget creation functions, we have
+ * to do it this way instead.  Fmeh.
+ */
+static GtkWidget *
+ui_custom_widget_handler(
+    GladeXML *xml,
+    gchar *func_name,
+    gchar *name,
+    gchar *string1,
+    gchar *string2,
+    gint int1,
+    gint int2,
+    gpointer user_data)
+{
+    int i;
+
+    for (i = 0 ; ui_callbacks[i].name ; i++)
+    {
+	if (!strcmp(func_name, ui_callbacks[i].name))
+	{
+	    /* TODO: pass down some of the string1 et al arguments */
+	    return ((GtkWidget *(*)(void))ui_callbacks[i].func)();
+	}
+    }
+    return 0;
+}
+
+/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
+
 #if GTK2
 #define LIBGLADE    "-glade2"
 #else
@@ -137,15 +186,17 @@ ui_load_tree(const char *root)
     /* load & create the interface */
     static char *filename = 0;
     static const char gladefile[] = PACKAGE LIBGLADE ".glade";
-    
+
     if (filename == 0)
     {
 	filename = find_file(ui_glade_path, gladefile);
 	if (filename == 0)
-	    fatal("can't find %s in path %s\n", gladefile, ui_glade_path);
+	    fatal("can't find %s in path %s\n", gladefile, ui_glade_path.data());
 	dprintf1(D_UICORE, "Loading Glade UI from file \"%s\"\n", filename);
     }
 
+    glade_set_custom_handler(ui_custom_widget_handler, (gpointer)0);
+
 #if GTK2
     xml = glade_xml_new(filename, root, /* translation domain */PACKAGE);
 #else
@@ -154,9 +205,9 @@ ui_load_tree(const char *root)
 
     if (xml == 0)
     	fatal("can't load Glade UI from file \"%s\"\n", filename);
-    
+
     /* connect the signals in the interface */
-    glade_xml_signal_autoconnect(xml);
+    ui_register_callbacks(xml);
 
     return xml;
 }
@@ -182,13 +233,15 @@ ui_get_dummy_menu(GladeXML *xml, const char *name)
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
-GtkWidget *ui_get_window(GtkWidget *w);
-
 static GtkWidget *
 _ui_virtual_parent(GtkWidget *w)
 {
     if (GTK_IS_MENU(w))
+#if GTK2
+	return gtk_menu_get_attach_widget(GTK_MENU(w));
+#else
     	return GTK_MENU(w)->parent_menu_item;
+#endif
     return GTK_WIDGET(w)->parent;
 }
 
