@@ -25,14 +25,8 @@
 #include <glade/glade.h>
 #include <gdk/gdkkeysyms.h>
 
-#ifdef __cplusplus
-#define GLADE_CALLBACK extern "C"
-#else
-#define GLADE_CALLBACK
-#endif
-
 GladeXML *ui_load_tree(const char *root);
-extern const char *ui_glade_path;
+void ui_prepend_glade_path(const char *dir);
 GtkWidget *ui_get_dummy_menu(GladeXML *xml, const char *name);
 
 
